const PARTICLE_OPTS = {
    direction: 'right',
    color: '#969696',
    type: 'triangle',
    easing: 'easeOutQuart',
    size: 2,
    particlesAmountCoefficient: 5,
    oscillationCoefficient: 3,
};

let gauntletEnabled = false;

// If the element is on screen somewhere
function isHidden (el) {
    return (
        el.offsetParent === null || 
        el.offsetWidth === 0 || 
        el.offsetHeight === 0
    )
}

// If the element is intended for a user, e.g., links, text, images
function isForUser (el) {
    if (el.text) {
        return true;
    }

    try {
        let elStyle = window.getComputedStyle(document.getElementById("hplogo"), null);
        let bg = elStyle.getPropertyValue("background");

        if (bg.indexOf("url") >= 0) {
            return true;
        }

        let bgImage = elStyle.getPropertyValue("background-image");

        if (bgImage && bgImage !== "none") {
            return true;
        }
    } catch (e) {}

    return false;
}

function getLeafNodes (randPerc) {
    let nodes = Array.prototype.slice.call(document.querySelectorAll("*:not(html):not(head):not(script):not(meta)"), 0);
    let leafNodes = nodes.filter(function (elem) {
        // E.g., if randPerc is 50, we'll end up returning ~50% of all leaf nodes that are visible
        return !isHidden(elem) && isForUser(elem) && Math.random() * 100 < randPerc;
    });

    return leafNodes;
}

function disintegrate (node) {
    const particles = new Particles(node, PARTICLE_OPTS);
    particles.disintegrate();
}

chrome.runtime.onMessage.addListener(
  function (request, sender, sendResponse) {
    if (request.message === "clicked_snap") {
        let leafNodes = getLeafNodes(50); // Get approximately 50% of all leaf nodes
        leafNodes.forEach((leafNode, i) => {
            setTimeout(() => disintegrate(leafNode), i * 250);
        });
    } else if (request.message === "toggle_gauntlet") {
        gauntletEnabled = !gauntletEnabled;
    } else if (request.message === "toggle_gauntlet_start") {
        gauntletEnabled = true;
    }
  }
);

document.addEventListener("click", function (el) {
    if (gauntletEnabled) {
        el.preventDefault();
        disintegrate(el.target);
    }
});
